@@ -1,12 +1,13 @@
 package Class::Trigger;
-
+use 5.008_001;
 use strict;
 use vars qw($VERSION);
-$VERSION = "0.11_03";
+$VERSION = "0.13_01";
 
 use Carp ();
 
 my (%Triggers, %TriggerPoints);
+my %Fetch_All_Triggers_Cache;
 
 sub import {
     my $class = shift;
@@ -43,6 +44,11 @@ sub add_trigger {
     Carp::croak('add_trigger() needs coderef') unless ref($code) eq 'CODE';
     push @{ $triggers->{$when} }, [ $code, $abortable ];
 
+    # Clear the cache when class triggers are added.  Because triggers are 
+    # inherited adding a trigger to one class may effect others.  Simplest
+    # thing to do is to clear the whole thing.
+    %Fetch_All_Triggers_Cache = () unless ref $proto;
+
     1;
 }
 
@@ -81,9 +87,17 @@ sub call_trigger {
 }
 
 sub __fetch_all_triggers {
-    my ($obj, $when, $list, $order) = @_;
+    my ($obj, $when, $list, $order, $nocache) = @_;
+    $nocache = 0 unless defined $nocache;
     my $class = ref $obj || $obj;
     my $return;
+    my $when_key = defined $when ? $when : '';
+    
+    unless ($nocache) {
+        return __cached_triggers($obj, $when)
+            if $Fetch_All_Triggers_Cache{$class}{$when_key};
+    }
+    
     unless ($list) {
         # Absence of the $list parameter conditions the creation of
         # the unrolled list of triggers. These keep track of the unique
@@ -99,9 +113,10 @@ sub __fetch_all_triggers {
     push @classes, $class;
     foreach my $c (@classes) {
         next if $list->{$c};
-        if (UNIVERSAL::can($c, 'call_trigger')) {
+#        if (UNIVERSAL::can($c, 'call_trigger')) {
+        if ($c->can('call_trigger')) {
             $list->{$c} = [];
-            __fetch_all_triggers($c, $when, $list, $order)
+            __fetch_all_triggers($c, $when, $list, $order, 1)
                 unless $c eq $class;
             if (defined $when && $Triggers{$c}{$when}) {
                 push @$order, $c;
@@ -114,14 +129,34 @@ sub __fetch_all_triggers {
         foreach my $class (@$order) {
             push @triggers, @{ $list->{$class} };
         }
-        if (ref $obj && defined $when) {
-            my $obj_triggers = $obj->{__triggers}{$when};
-            push @triggers, @$obj_triggers if $obj_triggers;
-        }
-        return @triggers;
+
+        # Only cache the class triggers, object triggers would
+        # necessitate a much larger cache and they're cheap to
+        # get anyway.
+        $Fetch_All_Triggers_Cache{$class}{$when_key} = \@triggers;
+
+        return __cached_triggers($obj, $when);
     }
 }
 
+
+sub __cached_triggers {
+    my($proto, $when) = @_;
+    my $class = ref $proto || $proto;
+    
+    return @{ $Fetch_All_Triggers_Cache{$class}{$when || ''} },
+           @{ __object_triggers($proto, $when) };
+}
+
+
+sub __object_triggers {
+    my($obj, $when) = @_;
+    
+    return [] unless ref $obj && defined $when;
+    return $obj->{__triggers}{$when} || [];
+}
+
+
 sub __validate_triggerpoint {
     return unless my $points = $TriggerPoints{ref $_[0] || $_[0]};
     my ($self, $when) = @_;
@@ -339,6 +374,8 @@ Code by Tatsuhiko Miyagawa E<lt>miyagawa@bulknews.netE<gt>.
 Jesse Vincent added a code to get return values from triggers and
 abortable flag.
 
+=head1 LICENSE
+
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
